@@ -93,3 +93,56 @@ condition: filesize >= 20
 		t.Logf("simple iterator scan (aaa..bbb): %+v", mrs)
 	}
 }
+
+type testPanicIter struct {
+	data    []block
+	current int
+}
+
+func (it *testPanicIter) First() *MemoryBlock {
+	it.current = 0
+	return it.Next()
+}
+
+func (it *testPanicIter) Next() *MemoryBlock {
+	if it.current >= len(it.data) {
+		return nil
+	}
+	data := it.data[it.current].data
+	base := it.data[it.current].base
+	it.current += 1
+	return &MemoryBlock{
+		Base: base,
+		Size: uint64(len(data)),
+		FetchData: func(buf []byte) {
+			// Simulate a caught panic during data fetch
+			func() {
+				defer func() {
+					recover() // Recover from panic
+				}()
+				// Cause a panic by writing to a nil pointer
+				var nilPointer *int
+				*nilPointer = 0
+			}()
+			copy(buf, data)
+		},
+	}
+}
+
+func TestIteratorWithPanic(t *testing.T) {
+	rs := MustCompile(`
+import "math"
+rule t1 { condition: math.entropy(0, 10) < 0.5 }
+`, nil)
+	var mrs MatchRules
+	if err := rs.ScanMemBlocks(&testPanicIter{
+		data: []block{
+			{0, []byte("aaaaaaaaaaaaaaaa")},
+			{32, []byte("bbbbbbbbbbbbbbbb")},
+		},
+	}, 0, 0, &mrs); err != nil {
+		t.Errorf("simple iterator scan (aaa..bbbb): %v", err)
+	} else {
+		t.Logf("simple iterator scan (aaa..bbb): %+v", mrs)
+	}
+}
